@@ -1,4 +1,4 @@
-# Copyright 2009-2020 Richard Dymond (rjdymond@gmail.com)
+# Copyright 2009-2022 Richard Dymond (rjdymond@gmail.com)
 #
 # This file is part of SkoolKit.
 #
@@ -82,6 +82,7 @@ def _parse_length(length, default_base, required):
 class CtlParser:
     def __init__(self, ctls=None):
         self._subctls = {}
+        self._ends = {}
         self._titles = {}
         self._instruction_comments = {}
         self._descriptions = defaultdict(list)
@@ -135,6 +136,7 @@ class CtlParser:
                         self._asm_data_directives[start].append(text[1:])
                 elif ctl.islower():
                     self._titles[start] = comment
+                    self._ends[start] = (ctl, 0)
                 elif ctl == 'D':
                     self._descriptions[start].append(comment)
                     self._subctls.setdefault(start, None)
@@ -146,7 +148,11 @@ class CtlParser:
                 elif ctl == 'R':
                     self._registers[start].append(comment)
                 elif ctl == 'M':
-                    self._multiline_comments[start] = (end, comment)
+                    if lengths:
+                        repeat = lengths[0][0]
+                    else:
+                        repeat = 0
+                    self._multiline_comments[start] = (end, comment, repeat)
                     self._subctls.setdefault(start, None)
                 elif ctl == 'L':
                     count = lengths[0][0]
@@ -165,7 +171,8 @@ class CtlParser:
                     self._instruction_comments[start] = comment
                     if end:
                         self._subctls[end] = None
-                if ctl != 'L' and lengths:
+                        self._ends[start] = (ctl, end)
+                if ctl in 'BCSTW' and lengths:
                     self._lengths[start] = lengths[0][1]
                     if len(lengths) > 1:
                         address = start + lengths[0][0]
@@ -174,11 +181,12 @@ class CtlParser:
                             self._lengths[address] = sublengths
                             self._subctls[address] = subctl
                             address += length
-                        self._multiline_comments[start] = (address, comment)
+                        self._multiline_comments.setdefault(start, (address, comment, 0))
             elif asm_directive:
                 directive, address = asm_directive
                 self._asm_directives[address].append(directive)
 
+        self._check_overlaps()
         self._terminate_multiline_comments()
         self._unroll_loops(max_address)
         self._ctls[max_address] = 'i'
@@ -270,12 +278,19 @@ class CtlParser:
             raise CtlParserError("invalid @ignoreua directive suffix: '{}'".format(comment_type))
         self._ignoreua_directives[address][comment_type] = suffix
 
+    def _check_overlaps(self):
+        subctls = sorted(self._ends.items())
+        for i, (s2, (c2, e2)) in enumerate(subctls[1:]):
+            s1, (c1, e1) = subctls[i]
+            if c1.isupper() and e1 > s2:
+                warn(f"'{c1}' directive at {s1}/${s1:04X} overlaps '{c2}' directive at {s2}/${s2:04X}")
+
     def _terminate_multiline_comments(self):
         addresses = sorted(set(self._ctls) | set(self._mid_block_comments) | {65536})
-        for address, (end, text) in self._multiline_comments.items():
+        for address, (end, text, repeat) in self._multiline_comments.items():
             max_end = addresses[bisect.bisect_right(addresses, address)]
             if end is None or end > max_end:
-                self._multiline_comments[address] = (max_end, text)
+                self._multiline_comments[address] = (max_end, text, repeat)
 
     def _unroll_loops(self, max_address):
         for start, end, count, repeat_entries in self._loops:
@@ -300,12 +315,12 @@ class CtlParser:
     def _repeat_multiline_comments(self, start, end, count, max_address):
         interval = end - start
         repeated = {k: v for k, v in self._multiline_comments.items() if start <= k < end}
-        for addr, (mlc_end, comment) in repeated.items():
+        for addr, (mlc_end, comment, repeat) in repeated.items():
             for i in range(1, count):
                 offset = i * interval
                 address = addr + offset
                 if address < max_address:
-                    self._multiline_comments[address] = (mlc_end + offset, comment)
+                    self._multiline_comments[address] = (mlc_end + offset, comment, repeat)
                 else:
                     break
 
@@ -356,8 +371,14 @@ class CtlParser:
                 sub_address = sub_block.start
                 sub_block.sublengths = self._lengths.get(sub_address, ((0, BASE_MAP[sub_block.ctl.upper()]),))
                 sub_block.header = self._reduce(self._mid_block_comments, sub_address)
-                sub_block.comment = self._instruction_comments.get(sub_address) or ()
-                sub_block.multiline_comment = self._multiline_comments.get(sub_address)
+                sub_block.comment = (self._instruction_comments.get(sub_address) or ())[:]
+                m_comment = self._multiline_comments.get(sub_address)
+                if m_comment:
+                    sub_block.multiline_comment = m_comment[:2]
+                    sub_block.repeat_comment = m_comment[2]
+                else:
+                    sub_block.multiline_comment = None
+                    sub_block.repeat_comment = 0
                 sub_block.asm_directives = dict([d for d in asm_directives if sub_address <= d[0] < sub_block.end])
                 sub_block.ignoreua_directives = {}
                 for addr, dirs in self._ignoreua_directives.items():
