@@ -1,4 +1,4 @@
-# Copyright 2010-2015, 2017-2019 Richard Dymond (rjdymond@gmail.com)
+# Copyright 2010-2015, 2017-2019, 2021 Richard Dymond (rjdymond@gmail.com)
 #
 # This file is part of SkoolKit.
 #
@@ -25,6 +25,7 @@ class OperandFormatter:
                    * `asm_hex` - if `True`, default base is hexadecimal
                    * `asm_lower` - if `True`, format operands in lower case
     """
+    # Component API
     def __init__(self, config):
         self.byte_formats = {
             'b': '%{:08b}',
@@ -49,6 +50,7 @@ class OperandFormatter:
             self.byte_formats[DEFAULT_BASE] = self.byte_formats['h']
             self.word_formats[DEFAULT_BASE] = self.word_formats['h']
 
+    # Component API
     def format_byte(self, value, base):
         """Format a byte value.
 
@@ -58,6 +60,7 @@ class OperandFormatter:
         """
         return self._num_str(value, 1, base)
 
+    # Component API
     def format_word(self, value, base):
         """Format a word (2-byte) value.
 
@@ -67,6 +70,7 @@ class OperandFormatter:
         """
         return self._num_str(value, 2, base)
 
+    # Component API
     def is_char(self, value):
         """Return whether a byte value can be formatted as a character.
 
@@ -108,12 +112,16 @@ class Disassembler:
                      DEFM statement
                    * `defw_size` - default maximum number of words in a DEFW
                      statement
+                   * `wrap` - if `True`, disassemble an instruction that wraps
+                     around the 64K boundary
     """
+    # Component API
     def __init__(self, snapshot, config):
         self.snapshot = snapshot
         self.defb_size = config.defb_size
         self.defm_size = config.defm_size
         self.defw_size = config.defw_size
+        self.wrap = config.wrap
         self.op_formatter = get_component('OperandFormatter', config)
         self.defb = 'DEFB '
         self.defm = 'DEFM '
@@ -130,6 +138,7 @@ class Disassembler:
             self.after_ED = {k: (v[0], v[1].lower()) for k, v in self.after_ED.items()}
             self.after_DDCB = {k: (v[0], v[1].lower()) for k, v in self.after_DDCB.items()}
 
+    # Component API
     def disassemble(self, start, end, base):
         """Disassemble an address range.
 
@@ -151,6 +160,8 @@ class Disassembler:
                 operation, length = decoder(self, template, address, base)
             if address + length <= 65536:
                 instructions.append((address, operation, self.snapshot[address:address + length]))
+            elif self.wrap:
+                instructions.append((address, operation, self.snapshot[address:65536] + self.snapshot[:(address + length) & 65535]))
             else:
                 instructions.append(self._defb_line(address, self.snapshot[address:65536]))
             address += length
@@ -177,6 +188,7 @@ class Disassembler:
             instructions.append(self._defb_line(i - len(data) + 1, data, sublengths, defm))
         return instructions
 
+    # Component API
     def defb_range(self, start, end, sublengths):
         """Produce a sequence of DEFB statements for an address range.
 
@@ -187,6 +199,7 @@ class Disassembler:
         """
         return self._defb_lines(start, end, sublengths)
 
+    # Component API
     def defm_range(self, start, end, sublengths):
         """Produce a sequence of DEFM statements for an address range.
 
@@ -214,6 +227,7 @@ class Disassembler:
             instructions.insert(0, (start, self.defw + ','.join(items), data))
         return instructions
 
+    # Component API
     def defw_range(self, start, end, sublengths):
         """Produce a sequence of DEFW statements for an address range.
 
@@ -234,6 +248,7 @@ class Disassembler:
             instructions.extend(self._defw_lines(address, address + size, ((size, base),)))
         return instructions
 
+    # Component API
     def defs_range(self, start, end, sublengths):
         """Produce a sequence of DEFS statements for an address range.
 
@@ -318,7 +333,8 @@ class Disassembler:
         return directive + self.defb_items(data, sublengths)
 
     def _defb(self, a, length):
-        return self.defb_dir(self.snapshot[a:a + length]), length
+        data = self.snapshot[a:a + length]
+        return self.defb_dir(data), len(data)
 
     def defb4(self, a, base):
         return self._defb(a, 4)
