@@ -1,4 +1,4 @@
-# Copyright 2012-2014, 2016-2017, 2019-2020 Richard Dymond (rjdymond@gmail.com)
+# Copyright 2012-2014, 2016-2017, 2019-2021 Richard Dymond (rjdymond@gmail.com)
 #
 # This file is part of SkoolKit.
 #
@@ -103,7 +103,7 @@ class PngWriter:
         for frame in frames[1:]:
             frame_data = self._build_image_data(frame, palette_size, bit_depth, attr_map)[0]
             seq_num += 1
-            self._write_fctl_chunk(img_file, seq_num, frame.delay, frame.width, frame.height)
+            self._write_fctl_chunk(img_file, seq_num, frame.delay, frame.width, frame.height, frame.x_offset, frame.y_offset)
             seq_num += 1
             fdat = FDAT + bytes(self._to_bytes(seq_num))
             self._write_img_data_chunk(img_file, fdat + frame_data)
@@ -305,7 +305,7 @@ class PngWriter:
             )
             pixel_rows = ([], [], [], [], [], [], [], [])
             for udg in row[c0:c1]:
-                paper, ink = attrs[udg.attr & 127]
+                paper, ink = attrs[udg.attr]
                 for k in range(k0, k1):
                     b = mask.apply(udg, k, pixels[paper], pixels[ink], trans)
                     p = pixel_rows[k]
@@ -336,7 +336,7 @@ class PngWriter:
         return img_data
 
     def _scan_udg_bd4_nt(self, udg, scanlines, attrs):
-        pixels = attrs[udg.attr & 127]
+        pixels = attrs[udg.attr]
         udg_bytes = udg.data
         byte = udg_bytes[0]
         scanlines[0].extend(pixels[byte // 16])
@@ -370,7 +370,7 @@ class PngWriter:
         return self._scan_frame(frame, self._scan_udg_bd4_nt, attrs)
 
     def _scan_udg_bd2_nt(self, udg, scanlines, attrs):
-        pixels = attrs[udg.attr & 127]
+        pixels = attrs[udg.attr]
         udg_bytes = udg.data
         byte = udg_bytes[0]
         scanlines[0].extend(pixels[byte // 16])
@@ -404,7 +404,7 @@ class PngWriter:
         return self._scan_frame(frame, self._scan_udg_bd2_nt, attrs)
 
     def _scan_udg_bd2_at(self, udg, scanlines, attrs):
-        pixels = attrs[udg.attr & 127]
+        pixels = attrs[udg.attr]
         udg_bytes = udg.data
         mask_bytes = udg.mask or udg_bytes
         byte = udg_bytes[0]
@@ -450,7 +450,7 @@ class PngWriter:
         return self._scan_frame(frame, self._scan_udg_bd2_at, attrs)
 
     def _scan_udg_bd1_nt(self, udg, scanlines, attrs, bits):
-        paper, ink = attrs[udg.attr & 127]
+        paper, ink = attrs[udg.attr]
         b_mask = paper * 255
         if ink == paper:
             data = bits[b_mask]
@@ -479,7 +479,7 @@ class PngWriter:
         return self._scan_frame(frame, self._scan_udg_bd1_nt, frame.attr_map, bits)
 
     def _scan_udg_bd1_at(self, udg, scanlines, mask, attrs, pixels, bits):
-        p, i = attrs[udg.attr & 127]
+        p, i = attrs[udg.attr]
         paper, ink = pixels[p], pixels[i]
         scanlines[0].extend(bits[int(''.join(mask.apply(udg, 0, paper, ink, '0')), 2)])
         scanlines[1].extend(bits[int(''.join(mask.apply(udg, 1, paper, ink, '0')), 2)])
