@@ -1,4 +1,4 @@
-# Copyright 2008-2020 Richard Dymond (rjdymond@gmail.com)
+# Copyright 2008-2022 Richard Dymond (rjdymond@gmail.com)
 #
 # This file is part of SkoolKit.
 #
@@ -167,7 +167,7 @@ def parse_ref_files(reffiles, ref_parser, fnames, search_dir=''):
                 reffiles.append(ref_f)
                 ref_parser.parse(ref_f)
 
-def run(infiles, options):
+def run(infiles, options, config):
     ref_search_dir = module_path = ''
     skoolfile = infiles[0]
     if skoolfile == '-':
@@ -187,12 +187,12 @@ def run(infiles, options):
         reffiles.insert(0, main_ref)
     ref_parser = RefParser()
     ref_parser.parse(StringIO(defaults.get_section('Config')))
-    config = ref_parser.get_dictionary('Config')
+    ref_config = ref_parser.get_dictionary('Config')
     for oreffile_f in reffiles:
         ref_parser.parse(oreffile_f)
     add_lines(ref_parser, options.config_specs, 'Config')
-    config.update(ref_parser.get_dictionary('Config'))
-    parse_ref_files(reffiles, ref_parser, config.get('RefFiles', '').split(';'), ref_search_dir)
+    ref_config.update(ref_parser.get_dictionary('Config'))
+    parse_ref_files(reffiles, ref_parser, ref_config.get('RefFiles', '').split(';'), ref_search_dir)
     parse_ref_files(reffiles, ref_parser, infiles[1:])
     add_lines(ref_parser, options.config_specs)
 
@@ -205,18 +205,21 @@ def run(infiles, options):
     elif skoolfile != '-':
         notify('Found no ref file for ' + normpath(skoolfile))
 
-    html_writer_class = get_object(config['HtmlWriterClass'], module_path)
-    game_dir = config.get('GameDir', prefix)
+    if 'InitModule' in ref_config:
+        get_object(ref_config['InitModule'], module_path)
+    html_writer_class = get_object(ref_config['HtmlWriterClass'], module_path)
+    game_dir = ref_config.get('GameDir', prefix)
+    label_fmt = (config['EntryLabel'], config['EntryPointLabel'])
 
     # Parse the skool file and initialise the writer
     skool_parser = clock(SkoolParser, 'Parsing {}'.format(fname), skoolfile, case=options.case, base=options.base,
-                         html=True, create_labels=options.create_labels, asm_labels=options.asm_labels,
+                         html=True, create_labels=options.create_labels, asm_labels=options.asm_labels, label_fmt=label_fmt,
                          variables=options.variables)
     if options.output_dir == '.':
         topdir = ''
     else:
         topdir = normpath(options.output_dir)
-    file_info = FileInfo(topdir, game_dir, options.new_images)
+    file_info = FileInfo(topdir, game_dir, options.new_images, options.new_audio)
     html_writer = html_writer_class(skool_parser, ref_parser, file_info)
 
     # Check that the specified pages exist
@@ -336,6 +339,8 @@ def main(args):
                        help="Write the disassembly in lower case.")
     group.add_argument('-o', '--rebuild-images', dest='new_images', action='store_const', const=1, default=config['RebuildImages'],
                        help="Overwrite existing image files.")
+    group.add_argument('-O', '--rebuild-audio', dest='new_audio', action='store_const', const=1, default=config['RebuildAudio'],
+                       help="Overwrite existing audio files.")
     group.add_argument('-p', '--package-dir', dest='package_dir', action='store_true',
                        help="Show path to skoolkit package directory and exit.")
     group.add_argument('-P', '--pages', dest='pages', metavar='PAGES',
@@ -362,7 +367,7 @@ def main(args):
     group.add_argument('-u', '--upper', dest='case', action='store_const', const=CASE_UPPER, default=config['Case'],
                        help="Write the disassembly in upper case.")
     group.add_argument('--var', dest='variables', metavar='name=value', type=variable, action='append', default=[],
-                       help="Define a variable that can be used by @if and the SMPL\nmacros. This option may be used multiple times.")
+                       help="Define a variable that can be used by @if and skool\nmacros. This option may be used multiple times.")
     group.add_argument('-V', '--version', action='version',
                        version='SkoolKit {}'.format(VERSION),
                        help='Show SkoolKit version number and exit.')
@@ -389,7 +394,7 @@ def main(args):
         show_ref_sections(namespace.ref_sections)
     if unknown_args or not namespace.infiles:
         parser.exit(2, parser.format_help())
-    update_options('skool2html', namespace, namespace.params)
+    update_options('skool2html', namespace, namespace.params, config)
     verbose, show_timings = not namespace.quiet, namespace.show_timings
     if namespace.asm_one_page:
         namespace.config_specs.append('Game/AsmSinglePage=1')
@@ -399,6 +404,6 @@ def main(args):
         namespace.pages = namespace.pages.split(',')
     else:
         namespace.pages = []
-    run(namespace.infiles, namespace)
+    run(namespace.infiles, namespace, config)
     if show_timings:
         notify('Done ({0:0.2f}s)'.format(time.time() - start))
