@@ -24,7 +24,8 @@ from string import Template
 from skoolkit import (BASE_10, BASE_16, CASE_LOWER, CASE_UPPER, VERSION,
                       SkoolKitError, SkoolParsingError, eval_variable, evaluate)
 from skoolkit.graphics import Udg
-from skoolkit.simulator import Simulator
+from skoolkit.simulator import (Simulator, A, F, B, C, D, E, H, L, IXh, IXl, IYh, IYl,
+                                SP, I, R, xA, xF, xB, xC, xD, xE, xH, xL, PC, T)
 
 _map_cache = {}
 
@@ -101,10 +102,10 @@ class AudioTracer:
         self.spkr = None
         self.out_times = []
 
-    def write_port(self, simulator, port, value):
-        if port & 0xFF == 0xFE and self.spkr is None or self.spkr != value & 0x10:
+    def write_port(self, registers, port, value):
+        if port % 2 == 0 and self.spkr != value & 0x10:
             self.spkr = value & 0x10
-            self.out_times.append(simulator.tstates)
+            self.out_times.append(registers[T])
 
 # API
 def parse_ints(text, index=0, num=0, defaults=(), names=(), fields=None):
@@ -560,10 +561,8 @@ def parse_audio(writer, text, index, need_audio=None):
         if eval_delays:
             simulator = Simulator(writer.snapshot, config={'fast_djnz': True, 'fast_ldir': True})
             tracer = AudioTracer()
-            simulator.add_tracer(tracer)
-            simulator.run(start)
-            while simulator.pc != stop:
-                simulator.run()
+            simulator.set_tracer(tracer)
+            simulator.run(start, stop)
             delays = [t - tracer.out_times[i] for i, t in enumerate(tracer.out_times[1:])]
     else:
         if len(text) > end and text[end] == '(':
@@ -1148,22 +1147,28 @@ def parse_sim(writer, text, index, *cwd):
     tstates = registers.get('tstates', 0)
     simulator = Simulator(writer.snapshot, registers, {'tstates': tstates}, {'fast_djnz': True, 'fast_ldir': True})
     if start < 0:
-        start = simulator.pc
-    simulator.run(start)
-    while simulator.pc != stop:
-        simulator.run()
-    sim = {r: simulator.registers[r] for r in ('A', 'F', '^A', '^F', 'I', 'R', 'SP')}
-    sim['BC'] = simulator.registers['C'] + 256 * simulator.registers['B']
-    sim['DE'] = simulator.registers['E'] + 256 * simulator.registers['D']
-    sim['HL'] = simulator.registers['L'] + 256 * simulator.registers['H']
-    sim['^BC'] = simulator.registers['^C'] + 256 * simulator.registers['^B']
-    sim['^DE'] = simulator.registers['^E'] + 256 * simulator.registers['^D']
-    sim['^HL'] = simulator.registers['^L'] + 256 * simulator.registers['^H']
-    sim['IX'] = simulator.registers['IXl'] + 256 * simulator.registers['IXh']
-    sim['IY'] = simulator.registers['IYl'] + 256 * simulator.registers['IYh']
-    sim['PC'] = simulator.pc
-    sim['tstates'] = simulator.tstates
-    writer.fields['sim'] = sim
+        start = simulator.registers[PC]
+    simulator.run(start, stop)
+    sim_registers = simulator.registers
+    writer.fields['sim'] = {
+        'A': sim_registers[A],
+        'F': sim_registers[F],
+        'BC': sim_registers[C] + 256 * sim_registers[B],
+        'DE': sim_registers[E] + 256 * sim_registers[D],
+        'HL': sim_registers[L] + 256 * sim_registers[H],
+        '^A': sim_registers[xA],
+        '^F': sim_registers[xF],
+        '^BC': sim_registers[xC] + 256 * sim_registers[xB],
+        '^DE': sim_registers[xE] + 256 * sim_registers[xD],
+        '^HL': sim_registers[xL] + 256 * sim_registers[xH],
+        'IX': sim_registers[IXl] + 256 * sim_registers[IXh],
+        'IY': sim_registers[IYl] + 256 * sim_registers[IYh],
+        'I': sim_registers[I],
+        'R': sim_registers[R],
+        'SP': sim_registers[SP],
+        'PC': sim_registers[PC],
+        'tstates': sim_registers[T]
+    }
     return end, ''
 
 def parse_space(writer, text, index, *cwd):
@@ -1218,12 +1223,10 @@ def parse_tstates(writer, text, index, *cwd):
         if stop < 0:
             raise MacroParsingError(f"Missing stop address: '{text[index:end]}'")
         simulator = Simulator(writer.snapshot[:])
-        simulator.run(start)
-        while simulator.pc != stop:
-            simulator.run()
+        simulator.run(start, stop)
         if msg is None:
-            return end, str(simulator.tstates)
-        return end, Template(msg).safe_substitute(tstates=simulator.tstates)
+            return end, str(simulator.registers[T])
+        return end, Template(msg).safe_substitute(tstates=simulator.registers[T])
     if stop < start:
         stop = start + 1
     timings = writer.parser.get_instruction_timings(start, stop)
