@@ -1,4 +1,4 @@
-# Copyright 2013-2017, 2019-2022 Richard Dymond (rjdymond@gmail.com)
+# Copyright 2013-2017, 2019-2023 Richard Dymond (rjdymond@gmail.com)
 #
 # This file is part of SkoolKit.
 #
@@ -20,7 +20,7 @@ from skoolkit import SkoolKitError, get_dword, get_int_param, get_word, integer,
 from skoolkit.basic import BasicLister, VariableLister, get_char
 from skoolkit.config import get_config, show_config, update_options
 from skoolkit.opcodes import END, decode
-from skoolkit.snapshot import make_snapshot
+from skoolkit.snapshot import FRAME_DURATION, QFRAME_DURATION, make_snapshot
 from skoolkit.sna2skool import get_ctl_parser
 from skoolkit.snaskool import Disassembly
 
@@ -159,6 +159,12 @@ def _parse_z80(z80file):
     reg.border = (header[12] // 2) % 8
     reg.iff2 = header[28]
     reg.im = header[29] & 3
+    if version == 3:
+        t1 = (header[55] + 256 * header[56]) % QFRAME_DURATION
+        t2 = (2 - header[57]) % 4
+        reg.tstates = FRAME_DURATION - 1 - t2 * QFRAME_DURATION - t1
+    else:
+        reg.tstates = 0
 
     return header, reg, ram_blocks
 
@@ -274,6 +280,7 @@ def _parse_szx(szxfile):
             reg.r = block[25]
             reg.iff2 = block[27]
             reg.im = block[28]
+            reg.tstates = get_dword(block, 29)
         elif block_id == 'SPCR':
             reg.border = block[0]
 
@@ -372,6 +379,7 @@ def _parse_sna(snafile):
     reg.border = sna[26]
     reg.iff2 = (sna[19] & 4) // 4
     reg.im = sna[25]
+    reg.tstates = 0
 
     return sna[:27], reg, sna[27:]
 
@@ -454,7 +462,7 @@ def _call_graph(snapshot, ctlfiles, prefix, start, end, config):
             print('{} -> {{{}}}'.format(node_id, ' '.join(ref_ids)))
     print('}')
 
-def _find(snapshot, byte_seq, base_addr=16384):
+def _find(snapshot, byte_seq, base_addr):
     steps = '1'
     if '-' in byte_seq:
         byte_seq, steps = byte_seq.split('-', 1)
@@ -492,10 +500,10 @@ def _find_tile(snapshot, coords):
         print('|{:08b}|'.format(b).replace('0', ' ').replace('1', '*'))
     _find(snapshot, '{}-{}'.format(','.join([str(b) for b in byte_seq]), steps), 23296)
 
-def _find_text(snapshot, text):
+def _find_text(snapshot, text, base_addr):
     size = len(text)
     byte_values = [ord(c) for c in text]
-    for a in range(16384, 65536 - size + 1):
+    for a in range(base_addr, 65536 - size + 1):
         if snapshot[a:a + size] == byte_values:
             print("{0}-{1} {0:04X}-{1:04X}: {2}".format(a, a + size - 1, text))
 
@@ -517,11 +525,11 @@ def run(infile, options, config):
             options.word, options.basic, options.variables)):
         snapshot, start, end = make_snapshot(infile, options.org, page=options.page)
         if options.find:
-            _find(snapshot, options.find)
+            _find(snapshot, options.find, start)
         elif options.tile:
             _find_tile(snapshot, options.tile)
         elif options.text:
-            _find_text(snapshot, options.text)
+            _find_text(snapshot, options.text, start)
         elif options.call_graph:
             _call_graph(snapshot, options.ctlfiles, infile, start, end, config)
         elif options.peek:
