@@ -1,4 +1,4 @@
-# Copyright 2013, 2015-2018, 2020-2022 Richard Dymond (rjdymond@gmail.com)
+# Copyright 2013, 2015-2018, 2020-2023 Richard Dymond (rjdymond@gmail.com)
 #
 # This file is part of SkoolKit.
 #
@@ -17,14 +17,16 @@
 import sys
 import os
 import argparse
+import hashlib
 import tempfile
 import zipfile
 from urllib.request import Request, urlopen
 from urllib.parse import urlparse
 
 from skoolkit import (SkoolKitError, get_dword, get_int_param, get_object,
-                      get_word, get_word3, integer, open_file, read_bin_file,
-                      warn, write_line, ROM48, VERSION)
+                      get_word, get_word3, integer, open_file, parse_int,
+                      read_bin_file, warn, write_line, ROM48, VERSION)
+from skoolkit.loadsample import ACCELERATORS
 from skoolkit.loadtracer import LoadTracer
 from skoolkit.simulator import (Simulator, A, F, B, C, D, E, H, L, IXh, IXl, IYh, IYl,
                                 SP, I, R, xA, xF, xB, xC, xD, xE, xH, xL, PC)
@@ -153,6 +155,34 @@ SIM_LOAD_PATCH = {
     0x5CCD: 0x2222, # ""
     0x5CCF: 0x0D,   # ENTER
     0x5CD0: 0x80,   # End of program area
+    0xFF18: 0x0DF3, # Stack...
+    0xFF1A: 0x0BCE,
+    0xFF1C: 0x50E3,
+    0xFF1E: 0x0BCE,
+    0xFF20: 0x50E4,
+    0xFF22: 0x171D,
+    0xFF24: 0x0ADC,
+    0xFF26: 0x0BCE,
+    0xFF28: 0x50E7,
+    0xFF2A: 0x171A,
+    0xFF2C: 0x0ADC,
+    0xFF2E: 0x18D7,
+    0xFF30: 0x0038,
+    0xFF32: 0x0038,
+    0xFF34: 0x190D,
+    0xFF36: 0x5CCF,
+    0xFF38: 0x0306,
+    0xFF3A: 0x5C07,
+    0xFF3C: 0x004D,
+    0xFF3E: 0x33B1,
+    0xFF40: 0x5C92,
+    0xFF42: 0x0005,
+    0xFF44: 0x32C8,
+    0xFF46: 0x3429,
+    0xFF48: 0x369B,
+    0xFF4A: 0x001F,
+    0xFF4C: 0x1B6D,
+    0xFF4E: 0x1CDB,
     0xFF50: 0x1B52, # Address of SCAN-LOOP
     0xFF52: 0x1B76, # Address of STMT-RET
     0xFF54: 0x12B7, # Address of entry point in 'MAIN EXECUTION' loop at 0x12A2
@@ -167,31 +197,59 @@ SIM_LOAD_CODE_PATCH = {
     0x5C82: 0x14,   # ECHO-E
     0x5CCF: 0xAF,   # CODE
     0x5CD0: 0x0D,   # ENTER
-    0x5CD1: 0x80    # End of program area
+    0x5CD1: 0x80,   # End of program area
+    0xFF1C: 0x50EA, # Stack...
+    0xFF20: 0x50EB,
+    0xFF22: 0x1716,
+    0xFF28: 0x50EC,
+    0xFF2A: 0x1715,
+    0xFF30: 0x000A,
+    0xFF36: 0x5CD0
 }
 
 class SkoolKitArgumentParser(argparse.ArgumentParser):
     def convert_arg_line_to_args(self, arg_line):
-        for arg in arg_line.split():
-            if arg in (';', '#'):
+        args = []
+        arg = ''
+        q = None
+        for c in arg_line:
+            if c == q:
+                args.append(arg)
+                arg = ''
+                q = None
+            elif c in '\'"' and q is None:
+                q = c
+            elif c.isspace() and q is None:
+                if arg:
+                    args.append(arg)
+                    arg = ''
+            elif c in ';#' and q is None:
                 break
-            yield arg
+            else:
+                arg += c
+        if arg:
+            args.append(arg)
+        return args
 
 class TapeError(Exception):
     pass
 
 class TapeBlockTimings:
-    def __init__(self, pilot_len=0, pilot=0, sync=(), zero=0, one=0, pause=0):
+    def __init__(self, pilot_len=0, pilot=0, sync=(), zero=0, one=0, pause=0, used_bits=8):
         self.pilot_len = pilot_len
         self.pilot = pilot
         self.sync = sync
         self.zero = zero
         self.one = one
         self.pause = pause
+        self.used_bits = used_bits
 
-TAP_TIMINGS_HEADER = TapeBlockTimings(8063, 2168, (667, 735), 855, 1710, 3500000)
-
-TAP_TIMINGS_DATA = TapeBlockTimings(3223, 2168, (667, 735), 855, 1710, 3500000)
+def get_tape_block_timings(first_byte, pause=3500000):
+    if first_byte == 0:
+        # Header block
+        return TapeBlockTimings(8063, 2168, (667, 735), 855, 1710, pause)
+    # Data block
+    return TapeBlockTimings(3223, 2168, (667, 735), 855, 1710, pause)
 
 def _write_z80(ram, options, fname):
     parent_dir = os.path.dirname(fname)
@@ -218,10 +276,43 @@ def _ram_operations(snapshot, ram_ops, blocks=None):
         else:
             raise SkoolKitError(f'Invalid operation: {spec}')
 
+def _set_sim_load_config(options):
+    options.accelerator = None
+    options.fast_load = True
+    options.first_edge = -2168
+    options.pause = True
+    options.timeout = 900
+    options.trace = None
+    for spec in options.sim_load_config:
+        name, sep, value = spec.lower().partition('=')
+        if sep:
+            if name == 'accelerator':
+                options.accelerator = value
+            elif name == 'fast-load': # pragma: no cover
+                options.fast_load = parse_int(value, options.fast_load)
+            elif name == 'first-edge': # pragma: no cover
+                options.first_edge = parse_int(value, options.first_edge)
+            elif name == 'pause': # pragma: no cover
+                options.pause = parse_int(value, options.pause)
+            elif name == 'timeout': # pragma: no cover
+                options.timeout = parse_int(value, options.timeout)
+            elif name == 'trace':
+                options.trace = value
+            else:
+                raise SkoolKitError(f'Invalid sim-load configuration parameter: {name}')
+
 def sim_load(blocks, options):
+    _set_sim_load_config(options)
+    if options.accelerator:
+        accelerator = ACCELERATORS.get(options.accelerator)
+        if options.accelerator != 'none' and accelerator is None:
+            raise SkoolKitError(f'Unrecognised accelerator: {options.accelerator}')
+    else:
+        accelerator = set(ACCELERATORS.values())
     snapshot = [0] * 65536
     rom = read_bin_file(ROM48, 16384)
     snapshot[:len(rom)] = rom
+    snapshot[0x5800:0x5B00] = [56] * 768 # PAPER 7: INK 0
     snapshot[0x5C00:0x5C00 + len(SYSVARS)] = SYSVARS
     for a, b in SIM_LOAD_PATCH.items():
         snapshot[a] = b % 256
@@ -234,12 +325,12 @@ def sim_load(blocks, options):
             if b > 0xFF:
                 snapshot[a + 1] = b // 256
     snapshot[0xFF58:] = snapshot[0x3E08:0x3EB0] # UDGs
-    config = {'fast_djnz': True, 'fast_ldir': True}
-    simulator = Simulator(snapshot, {'SP': 0xFF50}, config=config)
-    tracer = LoadTracer(blocks)
+    simulator = Simulator(snapshot, {'SP': 0xFF50})
+    tracer = LoadTracer(simulator, blocks, accelerator, options.pause, options.first_edge)
     simulator.set_tracer(tracer)
     try:
-        tracer.run(simulator, 0x0605, options.start) # SAVE-ETC
+        # Begin execution at 0x0605 (SAVE-ETC)
+        tracer.run(0x0605, options.start, options.fast_load, options.trace, options.timeout * 3500000)
         _ram_operations(snapshot, options.ram_ops)
     except KeyboardInterrupt: # pragma: no cover
         write_line(f'Simulation stopped (interrupted): PC={simulator.registers[PC]}')
@@ -269,7 +360,7 @@ def sim_load(blocks, options):
         'PC': sim_registers[PC]
     }
     options.reg = [f'{r}={v}' for r, v in registers.items()] + options.reg
-    state = [f'im={simulator.imode}', f'iff={simulator.iff2}', f'border={tracer.border}']
+    state = [f'im={simulator.imode}', f'iff={simulator.iff}', f'border={tracer.border}']
     options.state = state + options.state
     return simulator.memory[0x4000:]
 
@@ -377,13 +468,11 @@ def _get_tzx_block(data, i, sim):
     i += 1
     if block_id == 16:
         # Standard speed data block
+        pause = get_word(data, i) * 3500
         length = get_word(data, i + 2)
         tape_data = data[i + 4:i + 4 + length]
         if tape_data:
-            if tape_data[0] == 0:
-                timings = TAP_TIMINGS_HEADER
-            else:
-                timings = TAP_TIMINGS_DATA
+            timings = get_tape_block_timings(tape_data[0], pause)
         i += 4 + length
     elif block_id == 17:
         # Turbo speed data block
@@ -395,8 +484,9 @@ def _get_tzx_block(data, i, sim):
         zero = get_word(data, i + 6)
         one = get_word(data, i + 8)
         pilot_len = get_word(data, i + 10)
+        used_bits = data[i + 12]
         pause = get_word(data, i + 13) * 3500
-        timings = TapeBlockTimings(pilot_len, pilot, (sync1, sync2), zero, one, pause)
+        timings = TapeBlockTimings(pilot_len, pilot, (sync1, sync2), zero, one, pause, used_bits)
         i += 18 + length
     elif block_id == 18:
         # Pure tone
@@ -421,8 +511,9 @@ def _get_tzx_block(data, i, sim):
         if sim: # pragma: no cover
             zero = get_word(data, i)
             one = get_word(data, i + 2)
+            used_bits = data[i + 4]
             pause = get_word(data, i + 5) * 3500
-            timings = TapeBlockTimings(zero=zero, one=one, pause=pause)
+            timings = TapeBlockTimings(zero=zero, one=one, pause=pause, used_bits=used_bits)
         i += 10 + length
     elif block_id == 21:
         # Direct recording block
@@ -441,6 +532,9 @@ def _get_tzx_block(data, i, sim):
         i += get_dword(data, i) + 4
     elif block_id == 32:
         # Pause (silence) or 'Stop the tape' command
+        if sim: # pragma: no cover
+            pause = get_word(data, i) * 3500
+            timings = TapeBlockTimings(pause=pause)
         i += 2
     elif block_id == 33:
         # Group start
@@ -469,8 +563,6 @@ def _get_tzx_block(data, i, sim):
     elif block_id == 42:
         # Stop the tape if in 48K mode
         i += 4
-        if sim: # pragma: no cover
-            i = len(data)
     elif block_id == 43:
         # Set signal level
         i += 5
@@ -494,41 +586,65 @@ def _get_tzx_block(data, i, sim):
         i += 9
     else:
         raise TapeError('Unknown TZX block ID: 0x{:X}'.format(block_id))
-    return i, timings, tape_data
+    return i, block_id, timings, tape_data
 
-def _get_tzx_blocks(data, sim):
+def _get_tzx_blocks(data, sim, start, stop):
     signature = ''.join(chr(b) for b in data[:7])
     if signature != 'ZXTape!':
         raise TapeError("Not a TZX file")
     i = 10
     blocks = []
+    block_num = 1
+    loop = None
     while i < len(data):
-        i, timings, tape_data = _get_tzx_block(data, i, sim)
-        blocks.append((timings, tape_data))
+        if block_num >= stop > 0:
+            break
+        i, block_id, timings, tape_data = _get_tzx_block(data, i, sim)
+        if block_num >= start:
+            if sim: # pragma: no cover
+                if block_id == 0x20:
+                    if stop == 0 and timings.pause == 0:
+                        break
+                elif block_id == 0x24:
+                    loop = []
+                    repetitions = get_word(data, i - 2)
+                elif block_id == 0x2A and stop == 0:
+                    # Stop the tape if in 48K mode
+                    break
+            if loop is None:
+                blocks.append((timings, tape_data))
+            else: # pragma: no cover
+                loop.append((timings, tape_data))
+            if block_id == 0x25 and loop is not None: # pragma: no cover
+                blocks.extend(loop * repetitions)
+                loop = None
+        block_num += 1
     return blocks
 
-def get_tap_blocks(tap, sim=False):
+def get_tap_blocks(tap, start=1, stop=0):
     blocks = []
+    block_num = 1
     i = 0
     while i + 1 < len(tap):
+        if block_num >= stop > 0:
+            break # pragma: no cover (tested but missed by coverage)
         block_len = tap[i] + 256 * tap[i + 1]
         i += 2
-        data = tap[i:i + block_len]
-        if data:
-            if data[0] == 0:
-                timings = TAP_TIMINGS_HEADER
-            else:
-                timings = TAP_TIMINGS_DATA
-            blocks.append((timings, data))
+        if block_num >= start:
+            data = tap[i:i + block_len]
+            if data:
+                timings = get_tape_block_timings(data[0])
+                blocks.append((timings, data))
         i += block_len
+        block_num += 1
     return blocks
 
-def _get_tape_blocks(tape_type, tape, sim):
+def _get_tape_blocks(tape_type, tape, sim, start, stop):
     if tape_type.lower() == 'tzx':
-        return _get_tzx_blocks(tape, sim)
-    return get_tap_blocks(tape, sim)
+        return _get_tzx_blocks(tape, sim, start, stop)
+    return get_tap_blocks(tape, start, stop)
 
-def _get_tape(urlstring, user_agent, member=None):
+def _get_tape(urlstring, user_agent, member):
     url = urlparse(urlstring)
     if url.scheme:
         write_line('Downloading {0}'.format(urlstring))
@@ -554,7 +670,10 @@ def _get_tape(urlstring, user_agent, member=None):
                 f.close()
                 raise TapeError('No TAP or TZX file found')
         write_line('Extracting {0}'.format(member))
-        tape = z.open(member)
+        try:
+            tape = z.open(member)
+        except KeyError:
+            raise TapeError(f'No file named "{member}" in the archive')
         data = bytearray(tape.read())
         tape_type = member[-3:]
     else:
@@ -652,9 +771,73 @@ snapshot in an arbitrary way.
   suitable for a 48K ZX Spectrum.
 """.lstrip())
 
+def _print_sim_load_config_help():
+    width = max(len(n) for n in ACCELERATORS) + 2
+    names = [n.ljust(width) for n in sorted(ACCELERATORS)]
+    num_cols = 3
+    names += [''] * (-len(names) % num_cols)
+    cl = len(names) // num_cols
+    columns = [names[i:i + cl] for i in range(0, len(names), cl)]
+    accelerators = '\n  '.join(''.join(n) for n in zip(*columns))
+    print(f"""
+Usage: --sim-load-config accelerator=NAME
+       --sim-load-config fast-load=0/1
+       --sim-load-config first-edge=N
+       --sim-load-config pause=0/1
+       --sim-load-config timeout=N
+       --sim-load-config trace=FILE
+
+Use a specific tape-sampling loop accelerator, disable fast loading, set the
+location of the leading edge of the first pulse on the tape, disable pausing
+between tape blocks, set the timeout, or log executed instructions to a file.
+
+--sim-load-config accelerator=NAME
+
+  Use a specific accelerator to speed up the simulation of the tape-sampling
+  loop in a loading routine, or disable acceleration entirely. (By default, an
+  appropriate accelerator is automatically selected, if available.) Recognised
+  accelerator names are:
+
+  {accelerators}
+
+--sim-load-config fast-load=0/1
+
+  By default, whenever the Spectrum ROM's load routine is called, a shortcut is
+  taken by "fast loading" the next block on the tape. This significantly
+  reduces the load time for many tapes, but can also cause some loaders to
+  fail. Set fast-load=0 to disable fast loading.
+
+--sim-load-config first-edge=N
+
+  Set the time (in T-states) from the start of the tape at which to place the
+  leading edge of the first pulse (default: -2168). The default value places
+  the trailing edge of the first pulse at time 0, but some loaders (e.g.
+  polarity-sensitive loaders) require first-edge=0.
+
+--sim-load-config pause=0/1
+
+  By default, the tape is paused between blocks, and resumed whenever port 254
+  is read. While this can help with tapes that require (but do not actually
+  contain) long pauses between blocks, it can cause some loaders to fail. Set
+  pause=0 to disable this behaviour and run the tape continuously.
+
+--sim-load-config timeout=N
+
+  Set the timeout to N seconds (default: 900). A simulated LOAD still in
+  progress after this period of Z80 CPU time will automatically abort.
+
+--sim-load-config trace=FILE
+
+  Log to FILE all instructions executed during the simulated LOAD.
+""".strip())
+
 def make_z80(url, options, z80):
-    tape_type, tape = _get_tape(url, options.user_agent)
-    tape_blocks = _get_tape_blocks(tape_type, tape, options.sim_load)
+    tape_type, tape = _get_tape(url, options.user_agent, options.tape_name)
+    if options.tape_sum:
+        md5sum = hashlib.md5(tape).hexdigest()
+        if md5sum != options.tape_sum:
+            raise TapeError(f'Checksum mismatch: Expected {options.tape_sum}, actually {md5sum}')
+    tape_blocks = _get_tape_blocks(tape_type, tape, options.sim_load, options.tape_start, options.tape_stop)
     if options.sim_load:
         blocks = [b for b in tape_blocks if b[0]]
         ram = sim_load(blocks, options)
@@ -674,6 +857,9 @@ def main(args):
     )
     parser.add_argument('args', help=argparse.SUPPRESS, nargs='*')
     group = parser.add_argument_group('Options')
+    group.add_argument('-c', '--sim-load-config', metavar='name=value', action='append', default=[],
+                       help="Set the value of a --sim-load configuration parameter. "
+                            "Do '-c help' for more information. This option may be used multiple times.")
     group.add_argument('-d', '--output-dir', dest='output_dir', metavar='DIR',
                        help="Write the snapshot file in this directory.")
     group.add_argument('-f', '--force', action='store_true',
@@ -693,11 +879,22 @@ def main(args):
     group.add_argument('--state', dest='state', metavar='name=value', action='append', default=[],
                        help="Set a hardware state attribute. Do '--state help' for more information. "
                             "This option may be used multiple times.")
+    group.add_argument('--tape-name', metavar='NAME',
+                       help="Specify the name of a TAP/TZX file in a zip archive.")
+    group.add_argument('--tape-start', metavar='BLOCK', type=int, default=1,
+                       help="Start the tape at this block number.")
+    group.add_argument('--tape-stop', metavar='BLOCK', type=int, default=0,
+                       help="Stop the tape at this block number.")
+    group.add_argument('--tape-sum', metavar='MD5SUM',
+                       help="Specify the MD5 checksum of the TAP/TZX file.")
     group.add_argument('-u', '--user-agent', dest='user_agent', metavar='AGENT', default='',
                        help="Set the User-Agent header.")
     group.add_argument('-V', '--version', action='version', version='SkoolKit {}'.format(VERSION),
                        help='Show SkoolKit version number and exit.')
     namespace, unknown_args = parser.parse_known_args(args)
+    if 'help' in namespace.sim_load_config:
+        _print_sim_load_config_help()
+        return
     if 'help' in namespace.ram_ops:
         _print_ram_help()
         return
