@@ -1,4 +1,4 @@
-# Copyright 2013, 2015, 2017, 2020 Richard Dymond (rjdymond@gmail.com)
+# Copyright 2013, 2015, 2017, 2020, 2022 Richard Dymond (rjdymond@gmail.com)
 #
 # This file is part of SkoolKit.
 #
@@ -16,7 +16,7 @@
 
 import argparse
 
-from skoolkit import SkoolKitError, get_word, get_word3, get_dword, get_int_param, VERSION
+from skoolkit import SkoolKitError, get_word, get_word3, get_dword, get_int_param, warn, VERSION
 from skoolkit.basic import BasicLister, get_char
 
 ARCHIVE_INFO = {
@@ -264,11 +264,20 @@ def _get_block_info(data, i, block_num):
     i += 1
     if block_id == 16:
         header = 'Standard speed data'
+        info.append('Pause: {}ms'.format(get_word(data, i)))
         length = get_word(data, i + 2)
         tape_data = data[i + 4:i + 4 + length]
         i += 4 + length
     elif block_id == 17:
         header = 'Turbo speed data'
+        info.append('Pilot pulse: {}'.format(get_word(data, i)))
+        info.append('Sync pulse 1: {}'.format(get_word(data, i + 2)))
+        info.append('Sync pulse 2: {}'.format(get_word(data, i + 4)))
+        info.append('0-pulse: {}'.format(get_word(data, i + 6)))
+        info.append('1-pulse: {}'.format(get_word(data, i + 8)))
+        info.append('Pilot length: {} pulses'.format(get_word(data, i + 10)))
+        info.append('Used bits in last byte: {}'.format(data[i + 12]))
+        info.append('Pause: {}ms'.format(get_word(data, i + 13)))
         length = get_word3(data, i + 15)
         tape_data = data[i + 18:i + 18 + length]
         i += 18 + length
@@ -492,7 +501,7 @@ def _analyse_tzx(tzx, basic_block, options):
 def _analyse_tap(tap, basic_block, show_data):
     i = 0
     block_num = 1
-    while i < len(tap):
+    while i + 1 < len(tap):
         block_len = get_word(tap, i)
         data = tap[i + 2:i + 2 + block_len]
         if basic_block:
@@ -502,6 +511,11 @@ def _analyse_tap(tap, basic_block, show_data):
         i += block_len + 2
         block_num += 1
 
+    if i < len(tap):
+        warn('Extraneous byte at end of file')
+    elif i > len(tap):
+        warn(f'Missing {i - len(tap)} data byte(s) at end of file')
+
 def main(args):
     parser = argparse.ArgumentParser(
         usage="tapinfo.py FILE",
