@@ -55,18 +55,18 @@ def create_tap_data_block(data):
 def create_tap_header_block(title='', start=0, length=0, data_type=3):
     return [19, 0] + create_header_block(title, start, length, data_type)
 
-def create_tzx_data_block(data):
+def create_tzx_data_block(data, pause=0):
     block = [16] # Block ID
-    block.extend((0, 0)) # Pause duration
+    block.extend((pause % 256, pause // 256))
     data_block = create_data_block(data)
     length = len(data_block)
     block.extend((length % 256, length // 256))
     block.extend(data_block)
     return block
 
-def create_tzx_header_block(title='', start=0, length=0, data_type=3):
+def create_tzx_header_block(title='', start=0, length=0, data_type=3, pause=0):
     block = [16] # Block ID
-    block.extend((0, 0)) # Pause duration
+    block.extend((pause % 256, pause // 256))
     data_block = create_header_block(title, start, length, data_type)
     length = len(data_block)
     block.extend((length % 256, length // 256))
@@ -214,7 +214,7 @@ class SkoolKitTestCase(TestCase):
             block = []
             prev_b = None
             count = 0
-            for b in data + [-1]:
+            for b in data:
                 if b == prev_b or prev_b is None:
                     prev_b = b
                     if count < 255:
@@ -231,6 +231,10 @@ class SkoolKitTestCase(TestCase):
                     block += [prev_b] * count
                 prev_b = b
                 count = 1
+            if count > 4 or (count > 1 and prev_b == 237):
+                block.extend((237, 237, count, prev_b))
+            else:
+                block.extend((prev_b,) * count)
         else:
             block = data
         if page is not None:
@@ -254,6 +258,10 @@ class SkoolKitTestCase(TestCase):
         else:
             header[32] = pc % 256
             header[33] = pc // 256
+            if version == 3:
+                t = 69887 - (registers.get('tstates', 0) % 69888)
+                t1, t2 = t % 17472, t // 17472
+                header[55:58] = (t1 % 256, t1 // 256, (2 - t2) % 4)
         sp = registers.get('SP', 0)
         header[8] = sp % 256
         header[9] = sp // 256
