@@ -476,24 +476,30 @@ class SkoolMacroTest(SkoolKitTestCase):
 
     def test_parse_address_range(self):
         addr_specs = [
-            ('1', 1, [1]),
-            ('2x3', 1, [2] * 3),
-            ('0-3', 1, [0, 1, 2, 3]),
-            ('0-2x3', 1, [0, 1, 2] * 3),
-            ('0-6-2', 1, [0, 2, 4, 6]),
-            ('0-6-3x2', 1, [0, 3, 6] * 2),
-            ('0-49-1-16', 2, [0, 1, 16, 17, 32, 33, 48, 49]),
-            ('0-$0210-8-256x4', 3, [0, 8, 16, 256, 264, 272, 512, 520, 528] * 4),
+            ('1', 1, None, [1]),
+            ('2x3', 1, None, [2] * 3),
+            ('0-3', 1, None, [0, 1, 2, 3]),
+            ('0-2x3', 1, None, [0, 1, 2] * 3),
+            ('0-6-2', 1, None, [0, 2, 4, 6]),
+            ('0-6-3x2', 1, None, [0, 3, 6] * 2),
+            ('0-49-1-16', 2, None, [0, 1, 16, 17, 32, 33, 48, 49]),
+            ('0-$0210-8-256x4', 3, None, [0, 8, 16, 256, 264, 272, 512, 520, 528] * 4),
 
-            ('(1+3)', 1, [4]),
-            ('1-(1+1)', 2, [1, 2]),
-            ('1-(3-1)', 2, [1, 2]),
-            ('0-16-(4*2)', 3, [0, 8, 16]),
-            ('0-17-1-($08+8)', 2, [0, 1, 16, 17]),
-            ('1x(7/3)', 2, [1, 1]),
-            ('1x(1+$01)', 2, [1, 1])
+            ('(1+3)', 1, None, [4]),
+            ('1-(1+1)', 2, None, [1, 2]),
+            ('1-(3-1)', 2, None, [1, 2]),
+            ('0-16-(4*2)', 3, None, [0, 8, 16]),
+            ('0-17-1-($08+8)', 2, None, [0, 1, 16, 17]),
+            ('1x(7/3)', 2, None, [1, 1]),
+            ('1x(1+$01)', 2, None, [1, 1]),
+
+            ('({a})', 1, {'a': 0}, [0]),
+            ('0-({a})', 2, {'a': 1}, [0, 1]),
+            ('0-2-({a})', 2, {'a': 2}, [0, 2]),
+            ('0-24-1-({a})', 2, {'a': 16}, [0, 1, 16, 17]),
+            ('0-24-1-16x({a})', 2, {'a': 1}, [0, 1, 16, 17])
         ]
-        for spec, width, exp_addresses in addr_specs:
-            end, addresses = parse_address_range(spec, 0, width)
+        for spec, width, fields, exp_addresses in addr_specs:
+            end, addresses = parse_address_range(spec, 0, width, fields)
             self.assertEqual(end, len(spec), spec)
             self.assertEqual(exp_addresses, addresses)
