@@ -2,7 +2,7 @@ from unittest.mock import patch
 
 from skoolkittest import SkoolKitTestCase
 from skoolkit import components
-from skoolkit.z80 import Assembler
+from skoolkit.z80 import Assembler, get_timing
 
 OPERATIONS = (
     ('ADC A,244', (206, 244)),
@@ -1402,6 +1402,812 @@ DEFS = (
     ('2,-%10011001', (103, 103))
 )
 
+TIMINGS = (
+    ("ADC A,0", (206, 0), 7),
+    ("ADC A,B", (136,), 4),
+    ("ADC A,C", (137,), 4),
+    ("ADC A,D", (138,), 4),
+    ("ADC A,E", (139,), 4),
+    ("ADC A,H", (140,), 4),
+    ("ADC A,L", (141,), 4),
+    ("ADC A,(HL)", (142,), 7),
+    ("ADC A,A", (143,), 4),
+    ("ADC A,IXh", (221, 140), 8),
+    ("ADC A,IXl", (221, 141), 8),
+    ("ADC A,(IX+0)", (221, 142, 0), 19),
+    ("ADC A,IYh", (253, 140), 8),
+    ("ADC A,IYl", (253, 141), 8),
+    ("ADC A,(IY+0)", (253, 142, 0), 19),
+    ("ADC HL,BC", (237, 74), 15),
+    ("ADC HL,DE", (237, 90), 15),
+    ("ADC HL,HL", (237, 106), 15),
+    ("ADC HL,SP", (237, 122), 15),
+    ("ADD A,0", (198, 0), 7),
+    ("ADD A,B", (128,), 4),
+    ("ADD A,C", (129,), 4),
+    ("ADD A,D", (130,), 4),
+    ("ADD A,E", (131,), 4),
+    ("ADD A,H", (132,), 4),
+    ("ADD A,L", (133,), 4),
+    ("ADD A,(HL)", (134,), 7),
+    ("ADD A,A", (135,), 4),
+    ("ADD A,IXh", (221, 132), 8),
+    ("ADD A,IXl", (221, 133), 8),
+    ("ADD A,(IX+0)", (221, 134, 0), 19),
+    ("ADD A,IYh", (253, 132), 8),
+    ("ADD A,IYl", (253, 133), 8),
+    ("ADD A,(IY+0)", (253, 134, 0), 19),
+    ("ADD HL,BC", (9,), 11),
+    ("ADD HL,DE", (25,), 11),
+    ("ADD HL,HL", (41,), 11),
+    ("ADD HL,SP", (57,), 11),
+    ("ADD IX,BC", (221, 9), 15),
+    ("ADD IX,DE", (221, 25), 15),
+    ("ADD IX,IX", (221, 41), 15),
+    ("ADD IX,SP", (221, 57), 15),
+    ("ADD IY,BC", (253, 9), 15),
+    ("ADD IY,DE", (253, 25), 15),
+    ("ADD IY,IY", (253, 41), 15),
+    ("ADD IY,SP", (253, 57), 15),
+    ("AND 0", (230, 0), 7),
+    ("AND B", (160,), 4),
+    ("AND C", (161,), 4),
+    ("AND D", (162,), 4),
+    ("AND E", (163,), 4),
+    ("AND H", (164,), 4),
+    ("AND L", (165,), 4),
+    ("AND (HL)", (166,), 7),
+    ("AND A", (167,), 4),
+    ("AND IXh", (221, 164), 8),
+    ("AND IXl", (221, 165), 8),
+    ("AND (IX+0)", (221, 166, 0), 19),
+    ("AND IYh", (253, 164), 8),
+    ("AND IYl", (253, 165), 8),
+    ("AND (IY+0)", (253, 166, 0), 19),
+    ("BIT 0,B", (203, 64), 8),
+    ("BIT 0,C", (203, 65), 8),
+    ("BIT 0,D", (203, 66), 8),
+    ("BIT 0,E", (203, 67), 8),
+    ("BIT 0,H", (203, 68), 8),
+    ("BIT 0,L", (203, 69), 8),
+    ("BIT 0,(HL)", (203, 70), 12),
+    ("BIT 0,A", (203, 71), 8),
+    ("BIT 0,(IX+0)", (221, 203, 0, 70), 20),
+    ("BIT 0,(IY+0)", (253, 203, 0, 70), 20),
+    ("BIT 1,B", (203, 72), 8),
+    ("BIT 1,C", (203, 73), 8),
+    ("BIT 1,D", (203, 74), 8),
+    ("BIT 1,E", (203, 75), 8),
+    ("BIT 1,H", (203, 76), 8),
+    ("BIT 1,L", (203, 77), 8),
+    ("BIT 1,(HL)", (203, 78), 12),
+    ("BIT 1,A", (203, 79), 8),
+    ("BIT 1,(IX+0)", (221, 203, 0, 78), 20),
+    ("BIT 1,(IY+0)", (253, 203, 0, 78), 20),
+    ("BIT 2,B", (203, 80), 8),
+    ("BIT 2,C", (203, 81), 8),
+    ("BIT 2,D", (203, 82), 8),
+    ("BIT 2,E", (203, 83), 8),
+    ("BIT 2,H", (203, 84), 8),
+    ("BIT 2,L", (203, 85), 8),
+    ("BIT 2,(HL)", (203, 86), 12),
+    ("BIT 2,A", (203, 87), 8),
+    ("BIT 2,(IX+0)", (221, 203, 0, 86), 20),
+    ("BIT 2,(IY+0)", (253, 203, 0, 86), 20),
+    ("BIT 3,B", (203, 88), 8),
+    ("BIT 3,C", (203, 89), 8),
+    ("BIT 3,D", (203, 90), 8),
+    ("BIT 3,E", (203, 91), 8),
+    ("BIT 3,H", (203, 92), 8),
+    ("BIT 3,L", (203, 93), 8),
+    ("BIT 3,(HL)", (203, 94), 12),
+    ("BIT 3,A", (203, 95), 8),
+    ("BIT 3,(IX+0)", (221, 203, 0, 94), 20),
+    ("BIT 3,(IY+0)", (253, 203, 0, 94), 20),
+    ("BIT 4,B", (203, 96), 8),
+    ("BIT 4,C", (203, 97), 8),
+    ("BIT 4,D", (203, 98), 8),
+    ("BIT 4,E", (203, 99), 8),
+    ("BIT 4,H", (203, 100), 8),
+    ("BIT 4,L", (203, 101), 8),
+    ("BIT 4,(HL)", (203, 102), 12),
+    ("BIT 4,A", (203, 103), 8),
+    ("BIT 4,(IX+0)", (221, 203, 0, 102), 20),
+    ("BIT 4,(IY+0)", (253, 203, 0, 102), 20),
+    ("BIT 5,B", (203, 104), 8),
+    ("BIT 5,C", (203, 105), 8),
+    ("BIT 5,D", (203, 106), 8),
+    ("BIT 5,E", (203, 107), 8),
+    ("BIT 5,H", (203, 108), 8),
+    ("BIT 5,L", (203, 109), 8),
+    ("BIT 5,(HL)", (203, 110), 12),
+    ("BIT 5,A", (203, 111), 8),
+    ("BIT 5,(IX+0)", (221, 203, 0, 110), 20),
+    ("BIT 5,(IY+0)", (253, 203, 0, 110), 20),
+    ("BIT 6,B", (203, 112), 8),
+    ("BIT 6,C", (203, 113), 8),
+    ("BIT 6,D", (203, 114), 8),
+    ("BIT 6,E", (203, 115), 8),
+    ("BIT 6,H", (203, 116), 8),
+    ("BIT 6,L", (203, 117), 8),
+    ("BIT 6,(HL)", (203, 118), 12),
+    ("BIT 6,A", (203, 119), 8),
+    ("BIT 6,(IX+0)", (221, 203, 0, 118), 20),
+    ("BIT 6,(IY+0)", (253, 203, 0, 118), 20),
+    ("BIT 7,B", (203, 120), 8),
+    ("BIT 7,C", (203, 121), 8),
+    ("BIT 7,D", (203, 122), 8),
+    ("BIT 7,E", (203, 123), 8),
+    ("BIT 7,H", (203, 124), 8),
+    ("BIT 7,L", (203, 125), 8),
+    ("BIT 7,(HL)", (203, 126), 12),
+    ("BIT 7,A", (203, 127), 8),
+    ("BIT 7,(IX+0)", (221, 203, 0, 126), 20),
+    ("BIT 7,(IY+0)", (253, 203, 0, 126), 20),
+    ("CALL 0", (205, 0, 0), 17),
+    ("CALL NZ,0", (196, 0, 0), (17, 10)),
+    ("CALL Z,0", (204, 0, 0), (17, 10)),
+    ("CALL NC,0", (212, 0, 0), (17, 10)),
+    ("CALL C,0", (220, 0, 0), (17, 10)),
+    ("CALL PO,0", (228, 0, 0), (17, 10)),
+    ("CALL PE,0", (236, 0, 0), (17, 10)),
+    ("CALL P,0", (244, 0, 0), (17, 10)),
+    ("CALL M,0", (252, 0, 0), (17, 10)),
+    ("CCF", (63,), 4),
+    ("CP 0", (254, 0), 7),
+    ("CP B", (184,), 4),
+    ("CP C", (185,), 4),
+    ("CP D", (186,), 4),
+    ("CP E", (187,), 4),
+    ("CP H", (188,), 4),
+    ("CP L", (189,), 4),
+    ("CP (HL)", (190,), 7),
+    ("CP A", (191,), 4),
+    ("CP IXh", (221, 188), 8),
+    ("CP IXl", (221, 189), 8),
+    ("CP (IX+0)", (221, 190, 0), 19),
+    ("CP IYh", (253, 188), 8),
+    ("CP IYl", (253, 189), 8),
+    ("CP (IY+0)", (253, 190, 0), 19),
+    ("CPD", (237, 169), 16),
+    ("CPDR", (237, 185), (21, 16)),
+    ("CPI", (237, 161), 16),
+    ("CPIR", (237, 177), (21, 16)),
+    ("CPL", (47,), 4),
+    ("DAA", (39,), 4),
+    ("DEC B", (5,), 4),
+    ("DEC C", (13,), 4),
+    ("DEC D", (21,), 4),
+    ("DEC E", (29,), 4),
+    ("DEC H", (37,), 4),
+    ("DEC L", (45,), 4),
+    ("DEC (HL)", (53,), 11),
+    ("DEC A", (61,), 4),
+    ("DEC IXh", (221, 37), 8),
+    ("DEC IXl", (221, 45), 8),
+    ("DEC (IX+0)", (221, 53, 0), 23),
+    ("DEC IYh", (253, 37), 8),
+    ("DEC IYl", (253, 45), 8),
+    ("DEC (IY+0)", (253, 53, 0), 23),
+    ("DEC BC", (11,), 6),
+    ("DEC DE", (27,), 6),
+    ("DEC HL", (43,), 6),
+    ("DEC SP", (59,), 6),
+    ("DEC IX", (221, 43), 10),
+    ("DEC IY", (253, 43), 10),
+    ("DI", (243,), 4),
+    ("DJNZ 0", (16, 254), (13, 8)),
+    ("EI", (251,), 4),
+    ("EX AF,AF'", (8,), 4),
+    ("EX DE,HL", (235,), 4),
+    ("EX (SP),HL", (227,), 19),
+    ("EX (SP),IX", (221, 227), 23),
+    ("EX (SP),IY", (253, 227), 23),
+    ("EXX", (217,), 4),
+    ("HALT", (118,), 4),
+    ("IM 0", (237, 70), 8),
+    ("IM 1", (237, 86), 8),
+    ("IM 2", (237, 94), 8),
+    ("IN A,(0)", (219, 0), 11),
+    ("IN B,(C)", (237, 64), 12),
+    ("IN C,(C)", (237, 72), 12),
+    ("IN D,(C)", (237, 80), 12),
+    ("IN E,(C)", (237, 88), 12),
+    ("IN H,(C)", (237, 96), 12),
+    ("IN L,(C)", (237, 104), 12),
+    ("IN A,(C)", (237, 120), 12),
+    ("INC B", (4,), 4),
+    ("INC C", (12,), 4),
+    ("INC D", (20,), 4),
+    ("INC E", (28,), 4),
+    ("INC H", (36,), 4),
+    ("INC L", (44,), 4),
+    ("INC (HL)", (52,), 11),
+    ("INC A", (60,), 4),
+    ("INC IXh", (221, 36), 8),
+    ("INC IXl", (221, 44), 8),
+    ("INC (IX+0)", (221, 52, 0), 23),
+    ("INC IYh", (253, 36), 8),
+    ("INC IYl", (253, 44), 8),
+    ("INC (IY+0)", (253, 52, 0), 23),
+    ("INC BC", (3,), 6),
+    ("INC DE", (19,), 6),
+    ("INC HL", (35,), 6),
+    ("INC SP", (51,), 6),
+    ("INC IX", (221, 35), 10),
+    ("INC IY", (253, 35), 10),
+    ("IND", (237, 170), 16),
+    ("INDR", (237, 186), (21, 16)),
+    ("INI", (237, 162), 16),
+    ("INIR", (237, 178), (21, 16)),
+    ("JP 0", (195, 0, 0), 10),
+    ("JP NZ,0", (194, 0, 0), 10),
+    ("JP Z,0", (202, 0, 0), 10),
+    ("JP NC,0", (210, 0, 0), 10),
+    ("JP C,0", (218, 0, 0), 10),
+    ("JP PO,0", (226, 0, 0), 10),
+    ("JP PE,0", (234, 0, 0), 10),
+    ("JP P,0", (242, 0, 0), 10),
+    ("JP M,0", (250, 0, 0), 10),
+    ("JP (HL)", (233,), 4),
+    ("JP (IX)", (221, 233), 8),
+    ("JP (IY)", (253, 233), 8),
+    ("JR 2", (24, 0), 12),
+    ("JR NZ,2", (32, 0), (12, 7)),
+    ("JR Z,2", (40, 0), (12, 7)),
+    ("JR NC,2", (48, 0), (12, 7)),
+    ("JR C,2", (56, 0), (12, 7)),
+    ("LD (0),A", (50, 0, 0), 13),
+    ("LD (0),BC", (237, 67, 0, 0), 20),
+    ("LD (0),DE", (237, 83, 0, 0), 20),
+    ("LD (0),HL", (34, 0, 0), 16),
+    ("LD (0),SP", (237, 115, 0, 0), 20),
+    ("LD (0),IX", (221, 34, 0, 0), 20),
+    ("LD (0),IY", (253, 34, 0, 0), 20),
+    ("LD A,(0)", (58, 0, 0), 13),
+    ("LD BC,(0)", (237, 75, 0, 0), 20),
+    ("LD DE,(0)", (237, 91, 0, 0), 20),
+    ("LD HL,(0)", (42, 0, 0), 16),
+    ("LD SP,(0)", (237, 123, 0, 0), 20),
+    ("LD IX,(0)", (221, 42, 0, 0), 20),
+    ("LD IY,(0)", (253, 42, 0, 0), 20),
+    ("LD BC,0", (1, 0, 0), 10),
+    ("LD DE,0", (17, 0, 0), 10),
+    ("LD HL,0", (33, 0, 0), 10),
+    ("LD SP,0", (49, 0, 0), 10),
+    ("LD SP,HL", (249,), 6),
+    ("LD SP,IX", (221, 249), 10),
+    ("LD SP,IY", (253, 249), 10),
+    ("LD IX,0", (221, 33, 0, 0), 14),
+    ("LD IY,0", (253, 33, 0, 0), 14),
+    ("LD (BC),A", (2,), 7),
+    ("LD A,(BC)", (10,), 7),
+    ("LD (DE),A", (18,), 7),
+    ("LD A,(DE)", (26,), 7),
+    ("LD I,A", (237, 71), 9),
+    ("LD A,I", (237, 87), 9),
+    ("LD R,A", (237, 79), 9),
+    ("LD A,R", (237, 95), 9),
+    ("LD B,0", (6, 0), 7),
+    ("LD B,B", (64,), 4),
+    ("LD B,C", (65,), 4),
+    ("LD B,D", (66,), 4),
+    ("LD B,E", (67,), 4),
+    ("LD B,H", (68,), 4),
+    ("LD B,L", (69,), 4),
+    ("LD B,(HL)", (70,), 7),
+    ("LD B,A", (71,), 4),
+    ("LD B,IXh", (221, 68), 8),
+    ("LD B,IXl", (221, 69), 8),
+    ("LD B,(IX+0)", (221, 70, 0), 19),
+    ("LD B,IYh", (253, 68), 8),
+    ("LD B,IYl", (253, 69), 8),
+    ("LD B,(IY+0)", (253, 70, 0), 19),
+    ("LD C,0", (14, 0), 7),
+    ("LD C,B", (72,), 4),
+    ("LD C,C", (73,), 4),
+    ("LD C,D", (74,), 4),
+    ("LD C,E", (75,), 4),
+    ("LD C,H", (76,), 4),
+    ("LD C,L", (77,), 4),
+    ("LD C,(HL)", (78,), 7),
+    ("LD C,A", (79,), 4),
+    ("LD C,IXh", (221, 76), 8),
+    ("LD C,IXl", (221, 77), 8),
+    ("LD C,(IX+0)", (221, 78, 0), 19),
+    ("LD C,IYh", (253, 76), 8),
+    ("LD C,IYl", (253, 77), 8),
+    ("LD C,(IY+0)", (253, 78, 0), 19),
+    ("LD D,0", (22, 0), 7),
+    ("LD D,B", (80,), 4),
+    ("LD D,C", (81,), 4),
+    ("LD D,D", (82,), 4),
+    ("LD D,E", (83,), 4),
+    ("LD D,H", (84,), 4),
+    ("LD D,L", (85,), 4),
+    ("LD D,(HL)", (86,), 7),
+    ("LD D,A", (87,), 4),
+    ("LD D,IXh", (221, 84), 8),
+    ("LD D,IXl", (221, 85), 8),
+    ("LD D,(IX+0)", (221, 86, 0), 19),
+    ("LD D,IYh", (253, 84), 8),
+    ("LD D,IYl", (253, 85), 8),
+    ("LD D,(IY+0)", (253, 86, 0), 19),
+    ("LD E,0", (30, 0), 7),
+    ("LD E,B", (88,), 4),
+    ("LD E,C", (89,), 4),
+    ("LD E,D", (90,), 4),
+    ("LD E,E", (91,), 4),
+    ("LD E,H", (92,), 4),
+    ("LD E,L", (93,), 4),
+    ("LD E,(HL)", (94,), 7),
+    ("LD E,A", (95,), 4),
+    ("LD E,IXh", (221, 92), 8),
+    ("LD E,IXl", (221, 93), 8),
+    ("LD E,(IX+0)", (221, 94, 0), 19),
+    ("LD E,IYh", (253, 92), 8),
+    ("LD E,IYl", (253, 93), 8),
+    ("LD E,(IY+0)", (253, 94, 0), 19),
+    ("LD H,0", (38, 0), 7),
+    ("LD H,B", (96,), 4),
+    ("LD H,C", (97,), 4),
+    ("LD H,D", (98,), 4),
+    ("LD H,E", (99,), 4),
+    ("LD H,H", (100,), 4),
+    ("LD H,L", (101,), 4),
+    ("LD H,(HL)", (102,), 7),
+    ("LD H,A", (103,), 4),
+    ("LD H,(IX+0)", (221, 102, 0), 19),
+    ("LD H,(IY+0)", (253, 102, 0), 19),
+    ("LD L,0", (46, 0), 7),
+    ("LD L,B", (104,), 4),
+    ("LD L,C", (105,), 4),
+    ("LD L,D", (106,), 4),
+    ("LD L,E", (107,), 4),
+    ("LD L,H", (108,), 4),
+    ("LD L,L", (109,), 4),
+    ("LD L,(HL)", (110,), 7),
+    ("LD L,A", (111,), 4),
+    ("LD L,(IX+0)", (221, 110, 0), 19),
+    ("LD L,(IY+0)", (253, 110, 0), 19),
+    ("LD (HL),0", (54, 0), 10),
+    ("LD (HL),B", (112,), 7),
+    ("LD (HL),C", (113,), 7),
+    ("LD (HL),D", (114,), 7),
+    ("LD (HL),E", (115,), 7),
+    ("LD (HL),H", (116,), 7),
+    ("LD (HL),L", (117,), 7),
+    ("LD (HL),A", (119,), 7),
+    ("LD A,0", (62, 0), 7),
+    ("LD A,B", (120,), 4),
+    ("LD A,C", (121,), 4),
+    ("LD A,D", (122,), 4),
+    ("LD A,E", (123,), 4),
+    ("LD A,H", (124,), 4),
+    ("LD A,L", (125,), 4),
+    ("LD A,(HL)", (126,), 7),
+    ("LD A,A", (127,), 4),
+    ("LD A,IXh", (221, 124), 8),
+    ("LD A,IXl", (221, 125), 8),
+    ("LD A,(IX+0)", (221, 126, 0), 19),
+    ("LD A,IYh", (253, 124), 8),
+    ("LD A,IYl", (253, 125), 8),
+    ("LD A,(IY+0)", (253, 126, 0), 19),
+    ("LD (IX+0),1", (221, 54, 0, 1), 19),
+    ("LD (IX+0),B", (221, 112, 0), 19),
+    ("LD (IX+0),C", (221, 113, 0), 19),
+    ("LD (IX+0),D", (221, 114, 0), 19),
+    ("LD (IX+0),E", (221, 115, 0), 19),
+    ("LD (IX+0),H", (221, 116, 0), 19),
+    ("LD (IX+0),L", (221, 117, 0), 19),
+    ("LD (IX+0),A", (221, 119, 0), 19),
+    ("LD IXh,0", (221, 38, 0), 11),
+    ("LD IXh,B", (221, 96), 8),
+    ("LD IXh,C", (221, 97), 8),
+    ("LD IXh,D", (221, 98), 8),
+    ("LD IXh,E", (221, 99), 8),
+    ("LD IXh,IXh", (221, 100), 8),
+    ("LD IXh,IXl", (221, 101), 8),
+    ("LD IXh,A", (221, 103), 8),
+    ("LD IXl,0", (221, 46, 0), 11),
+    ("LD IXl,B", (221, 104), 8),
+    ("LD IXl,C", (221, 105), 8),
+    ("LD IXl,D", (221, 106), 8),
+    ("LD IXl,E", (221, 107), 8),
+    ("LD IXl,IXh", (221, 108), 8),
+    ("LD IXl,IXl", (221, 109), 8),
+    ("LD IXl,A", (221, 111), 8),
+    ("LD (IY+0),1", (253, 54, 0, 1), 19),
+    ("LD (IY+0),B", (253, 112, 0), 19),
+    ("LD (IY+0),C", (253, 113, 0), 19),
+    ("LD (IY+0),D", (253, 114, 0), 19),
+    ("LD (IY+0),E", (253, 115, 0), 19),
+    ("LD (IY+0),H", (253, 116, 0), 19),
+    ("LD (IY+0),L", (253, 117, 0), 19),
+    ("LD (IY+0),A", (253, 119, 0), 19),
+    ("LD IYh,0", (253, 38, 0), 11),
+    ("LD IYh,B", (253, 96), 8),
+    ("LD IYh,C", (253, 97), 8),
+    ("LD IYh,D", (253, 98), 8),
+    ("LD IYh,E", (253, 99), 8),
+    ("LD IYh,IYh", (253, 100), 8),
+    ("LD IYh,IYl", (253, 101), 8),
+    ("LD IYh,A", (253, 103), 8),
+    ("LD IYl,0", (253, 46, 0), 11),
+    ("LD IYl,B", (253, 104), 8),
+    ("LD IYl,C", (253, 105), 8),
+    ("LD IYl,D", (253, 106), 8),
+    ("LD IYl,E", (253, 107), 8),
+    ("LD IYl,IYh", (253, 108), 8),
+    ("LD IYl,IYl", (253, 109), 8),
+    ("LD IYl,A", (253, 111), 8),
+    ("LDD", (237, 168), 16),
+    ("LDDR", (237, 184), (21, 16)),
+    ("LDI", (237, 160), 16),
+    ("LDIR", (237, 176), (21, 16)),
+    ("NEG", (237, 68), 8),
+    ("NOP", (0,), 4),
+    ("OR 0", (246, 0), 7),
+    ("OR B", (176,), 4),
+    ("OR C", (177,), 4),
+    ("OR D", (178,), 4),
+    ("OR E", (179,), 4),
+    ("OR H", (180,), 4),
+    ("OR L", (181,), 4),
+    ("OR (HL)", (182,), 7),
+    ("OR A", (183,), 4),
+    ("OR IXh", (221, 180), 8),
+    ("OR IXl", (221, 181), 8),
+    ("OR (IX+0)", (221, 182, 0), 19),
+    ("OR IYh", (253, 180), 8),
+    ("OR IYl", (253, 181), 8),
+    ("OR (IY+0)", (253, 182, 0), 19),
+    ("OTDR", (237, 187), (21, 16)),
+    ("OTIR", (237, 179), (21, 16)),
+    ("OUT (0),A", (211, 0), 11),
+    ("OUT (C),B", (237, 65), 12),
+    ("OUT (C),C", (237, 73), 12),
+    ("OUT (C),D", (237, 81), 12),
+    ("OUT (C),E", (237, 89), 12),
+    ("OUT (C),H", (237, 97), 12),
+    ("OUT (C),L", (237, 105), 12),
+    ("OUT (C),A", (237, 121), 12),
+    ("OUTD", (237, 171), 16),
+    ("OUTI", (237, 163), 16),
+    ("POP BC", (193,), 10),
+    ("POP DE", (209,), 10),
+    ("POP HL", (225,), 10),
+    ("POP AF", (241,), 10),
+    ("POP IX", (221, 225), 14),
+    ("POP IY", (253, 225), 14),
+    ("PUSH BC", (197,), 11),
+    ("PUSH DE", (213,), 11),
+    ("PUSH HL", (229,), 11),
+    ("PUSH AF", (245,), 11),
+    ("PUSH IX", (221, 229), 15),
+    ("PUSH IY", (253, 229), 15),
+    ("RES 0,B", (203, 128), 8),
+    ("RES 0,C", (203, 129), 8),
+    ("RES 0,D", (203, 130), 8),
+    ("RES 0,E", (203, 131), 8),
+    ("RES 0,H", (203, 132), 8),
+    ("RES 0,L", (203, 133), 8),
+    ("RES 0,(HL)", (203, 134), 15),
+    ("RES 0,A", (203, 135), 8),
+    ("RES 0,(IX+0)", (221, 203, 0, 134), 23),
+    ("RES 0,(IY+0)", (253, 203, 0, 134), 23),
+    ("RES 1,B", (203, 136), 8),
+    ("RES 1,C", (203, 137), 8),
+    ("RES 1,D", (203, 138), 8),
+    ("RES 1,E", (203, 139), 8),
+    ("RES 1,H", (203, 140), 8),
+    ("RES 1,L", (203, 141), 8),
+    ("RES 1,(HL)", (203, 142), 15),
+    ("RES 1,A", (203, 143), 8),
+    ("RES 1,(IX+0)", (221, 203, 0, 142), 23),
+    ("RES 1,(IY+0)", (253, 203, 0, 142), 23),
+    ("RES 2,B", (203, 144), 8),
+    ("RES 2,C", (203, 145), 8),
+    ("RES 2,D", (203, 146), 8),
+    ("RES 2,E", (203, 147), 8),
+    ("RES 2,H", (203, 148), 8),
+    ("RES 2,L", (203, 149), 8),
+    ("RES 2,(HL)", (203, 150), 15),
+    ("RES 2,A", (203, 151), 8),
+    ("RES 2,(IX+0)", (221, 203, 0, 150), 23),
+    ("RES 2,(IY+0)", (253, 203, 0, 150), 23),
+    ("RES 3,B", (203, 152), 8),
+    ("RES 3,C", (203, 153), 8),
+    ("RES 3,D", (203, 154), 8),
+    ("RES 3,E", (203, 155), 8),
+    ("RES 3,H", (203, 156), 8),
+    ("RES 3,L", (203, 157), 8),
+    ("RES 3,(HL)", (203, 158), 15),
+    ("RES 3,A", (203, 159), 8),
+    ("RES 3,(IX+0)", (221, 203, 0, 158), 23),
+    ("RES 3,(IY+0)", (253, 203, 0, 158), 23),
+    ("RES 4,B", (203, 160), 8),
+    ("RES 4,C", (203, 161), 8),
+    ("RES 4,D", (203, 162), 8),
+    ("RES 4,E", (203, 163), 8),
+    ("RES 4,H", (203, 164), 8),
+    ("RES 4,L", (203, 165), 8),
+    ("RES 4,(HL)", (203, 166), 15),
+    ("RES 4,A", (203, 167), 8),
+    ("RES 4,(IX+0)", (221, 203, 0, 166), 23),
+    ("RES 4,(IY+0)", (253, 203, 0, 166), 23),
+    ("RES 5,B", (203, 168), 8),
+    ("RES 5,C", (203, 169), 8),
+    ("RES 5,D", (203, 170), 8),
+    ("RES 5,E", (203, 171), 8),
+    ("RES 5,H", (203, 172), 8),
+    ("RES 5,L", (203, 173), 8),
+    ("RES 5,(HL)", (203, 174), 15),
+    ("RES 5,A", (203, 175), 8),
+    ("RES 5,(IX+0)", (221, 203, 0, 174), 23),
+    ("RES 5,(IY+0)", (253, 203, 0, 174), 23),
+    ("RES 6,B", (203, 176), 8),
+    ("RES 6,C", (203, 177), 8),
+    ("RES 6,D", (203, 178), 8),
+    ("RES 6,E", (203, 179), 8),
+    ("RES 6,H", (203, 180), 8),
+    ("RES 6,L", (203, 181), 8),
+    ("RES 6,(HL)", (203, 182), 15),
+    ("RES 6,A", (203, 183), 8),
+    ("RES 6,(IX+0)", (221, 203, 0, 182), 23),
+    ("RES 6,(IY+0)", (253, 203, 0, 182), 23),
+    ("RES 7,B", (203, 184), 8),
+    ("RES 7,C", (203, 185), 8),
+    ("RES 7,D", (203, 186), 8),
+    ("RES 7,E", (203, 187), 8),
+    ("RES 7,H", (203, 188), 8),
+    ("RES 7,L", (203, 189), 8),
+    ("RES 7,(HL)", (203, 190), 15),
+    ("RES 7,A", (203, 191), 8),
+    ("RES 7,(IX+0)", (221, 203, 0, 190), 23),
+    ("RES 7,(IY+0)", (253, 203, 0, 190), 23),
+    ("RET", (201,), 10),
+    ("RET NZ", (192,), (11, 5)),
+    ("RET Z", (200,), (11, 5)),
+    ("RET NC", (208,), (11, 5)),
+    ("RET C", (216,), (11, 5)),
+    ("RET PO", (224,), (11, 5)),
+    ("RET PE", (232,), (11, 5)),
+    ("RET P", (240,), (11, 5)),
+    ("RET M", (248,), (11, 5)),
+    ("RETI", (237, 77), 14),
+    ("RETN", (237, 69), 14),
+    ("RL B", (203, 16), 8),
+    ("RL C", (203, 17), 8),
+    ("RL D", (203, 18), 8),
+    ("RL E", (203, 19), 8),
+    ("RL H", (203, 20), 8),
+    ("RL L", (203, 21), 8),
+    ("RL (HL)", (203, 22), 15),
+    ("RL A", (203, 23), 8),
+    ("RL (IX+0)", (221, 203, 0, 22), 23),
+    ("RL (IY+0)", (253, 203, 0, 22), 23),
+    ("RLA", (23,), 4),
+    ("RLC B", (203, 0), 8),
+    ("RLC C", (203, 1), 8),
+    ("RLC D", (203, 2), 8),
+    ("RLC E", (203, 3), 8),
+    ("RLC H", (203, 4), 8),
+    ("RLC L", (203, 5), 8),
+    ("RLC (HL)", (203, 6), 15),
+    ("RLC A", (203, 7), 8),
+    ("RLC (IX+0)", (221, 203, 0, 6), 23),
+    ("RLC (IY+0)", (253, 203, 0, 6), 23),
+    ("RLCA", (7,), 4),
+    ("RLD", (237, 111), 18),
+    ("RR B", (203, 24), 8),
+    ("RR C", (203, 25), 8),
+    ("RR D", (203, 26), 8),
+    ("RR E", (203, 27), 8),
+    ("RR H", (203, 28), 8),
+    ("RR L", (203, 29), 8),
+    ("RR (HL)", (203, 30), 15),
+    ("RR A", (203, 31), 8),
+    ("RR (IX+0)", (221, 203, 0, 30), 23),
+    ("RR (IY+0)", (253, 203, 0, 30), 23),
+    ("RRA", (31,), 4),
+    ("RRC B", (203, 8), 8),
+    ("RRC C", (203, 9), 8),
+    ("RRC D", (203, 10), 8),
+    ("RRC E", (203, 11), 8),
+    ("RRC H", (203, 12), 8),
+    ("RRC L", (203, 13), 8),
+    ("RRC (HL)", (203, 14), 15),
+    ("RRC A", (203, 15), 8),
+    ("RRC (IX+0)", (221, 203, 0, 14), 23),
+    ("RRC (IY+0)", (253, 203, 0, 14), 23),
+    ("RRCA", (15,), 4),
+    ("RRD", (237, 103), 18),
+    ("RST 0", (199,), 11),
+    ("RST 8", (207,), 11),
+    ("RST 16", (215,), 11),
+    ("RST 24", (223,), 11),
+    ("RST 32", (231,), 11),
+    ("RST 40", (239,), 11),
+    ("RST 48", (247,), 11),
+    ("RST 56", (255,), 11),
+    ("SBC A,0", (222, 0), 7),
+    ("SBC A,B", (152,), 4),
+    ("SBC A,C", (153,), 4),
+    ("SBC A,D", (154,), 4),
+    ("SBC A,E", (155,), 4),
+    ("SBC A,H", (156,), 4),
+    ("SBC A,L", (157,), 4),
+    ("SBC A,(HL)", (158,), 7),
+    ("SBC A,A", (159,), 4),
+    ("SBC A,IXh", (221, 156), 8),
+    ("SBC A,IXl", (221, 157), 8),
+    ("SBC A,(IX+0)", (221, 158, 0), 19),
+    ("SBC A,IYh", (253, 156), 8),
+    ("SBC A,IYl", (253, 157), 8),
+    ("SBC A,(IY+0)", (253, 158, 0), 19),
+    ("SBC HL,BC", (237, 66), 15),
+    ("SBC HL,DE", (237, 82), 15),
+    ("SBC HL,HL", (237, 98), 15),
+    ("SBC HL,SP", (237, 114), 15),
+    ("SCF", (55,), 4),
+    ("SET 0,B", (203, 192), 8),
+    ("SET 0,C", (203, 193), 8),
+    ("SET 0,D", (203, 194), 8),
+    ("SET 0,E", (203, 195), 8),
+    ("SET 0,H", (203, 196), 8),
+    ("SET 0,L", (203, 197), 8),
+    ("SET 0,(HL)", (203, 198), 15),
+    ("SET 0,A", (203, 199), 8),
+    ("SET 0,(IX+0)", (221, 203, 0, 198), 23),
+    ("SET 0,(IY+0)", (253, 203, 0, 198), 23),
+    ("SET 1,B", (203, 200), 8),
+    ("SET 1,C", (203, 201), 8),
+    ("SET 1,D", (203, 202), 8),
+    ("SET 1,E", (203, 203), 8),
+    ("SET 1,H", (203, 204), 8),
+    ("SET 1,L", (203, 205), 8),
+    ("SET 1,(HL)", (203, 206), 15),
+    ("SET 1,A", (203, 207), 8),
+    ("SET 1,(IX+0)", (221, 203, 0, 206), 23),
+    ("SET 1,(IY+0)", (253, 203, 0, 206), 23),
+    ("SET 2,B", (203, 208), 8),
+    ("SET 2,C", (203, 209), 8),
+    ("SET 2,D", (203, 210), 8),
+    ("SET 2,E", (203, 211), 8),
+    ("SET 2,H", (203, 212), 8),
+    ("SET 2,L", (203, 213), 8),
+    ("SET 2,(HL)", (203, 214), 15),
+    ("SET 2,A", (203, 215), 8),
+    ("SET 2,(IX+0)", (221, 203, 0, 214), 23),
+    ("SET 2,(IY+0)", (253, 203, 0, 214), 23),
+    ("SET 3,B", (203, 216), 8),
+    ("SET 3,C", (203, 217), 8),
+    ("SET 3,D", (203, 218), 8),
+    ("SET 3,E", (203, 219), 8),
+    ("SET 3,H", (203, 220), 8),
+    ("SET 3,L", (203, 221), 8),
+    ("SET 3,(HL)", (203, 222), 15),
+    ("SET 3,A", (203, 223), 8),
+    ("SET 3,(IX+0)", (221, 203, 0, 222), 23),
+    ("SET 3,(IY+0)", (253, 203, 0, 222), 23),
+    ("SET 4,B", (203, 224), 8),
+    ("SET 4,C", (203, 225), 8),
+    ("SET 4,D", (203, 226), 8),
+    ("SET 4,E", (203, 227), 8),
+    ("SET 4,H", (203, 228), 8),
+    ("SET 4,L", (203, 229), 8),
+    ("SET 4,(HL)", (203, 230), 15),
+    ("SET 4,A", (203, 231), 8),
+    ("SET 4,(IX+0)", (221, 203, 0, 230), 23),
+    ("SET 4,(IY+0)", (253, 203, 0, 230), 23),
+    ("SET 5,B", (203, 232), 8),
+    ("SET 5,C", (203, 233), 8),
+    ("SET 5,D", (203, 234), 8),
+    ("SET 5,E", (203, 235), 8),
+    ("SET 5,H", (203, 236), 8),
+    ("SET 5,L", (203, 237), 8),
+    ("SET 5,(HL)", (203, 238), 15),
+    ("SET 5,A", (203, 239), 8),
+    ("SET 5,(IX+0)", (221, 203, 0, 238), 23),
+    ("SET 5,(IY+0)", (253, 203, 0, 238), 23),
+    ("SET 6,B", (203, 240), 8),
+    ("SET 6,C", (203, 241), 8),
+    ("SET 6,D", (203, 242), 8),
+    ("SET 6,E", (203, 243), 8),
+    ("SET 6,H", (203, 244), 8),
+    ("SET 6,L", (203, 245), 8),
+    ("SET 6,(HL)", (203, 246), 15),
+    ("SET 6,A", (203, 247), 8),
+    ("SET 6,(IX+0)", (221, 203, 0, 246), 23),
+    ("SET 6,(IY+0)", (253, 203, 0, 246), 23),
+    ("SET 7,B", (203, 248), 8),
+    ("SET 7,C", (203, 249), 8),
+    ("SET 7,D", (203, 250), 8),
+    ("SET 7,E", (203, 251), 8),
+    ("SET 7,H", (203, 252), 8),
+    ("SET 7,L", (203, 253), 8),
+    ("SET 7,(HL)", (203, 254), 15),
+    ("SET 7,A", (203, 255), 8),
+    ("SET 7,(IX+0)", (221, 203, 0, 254), 23),
+    ("SET 7,(IY+0)", (253, 203, 0, 254), 23),
+    ("SLA B", (203, 32), 8),
+    ("SLA C", (203, 33), 8),
+    ("SLA D", (203, 34), 8),
+    ("SLA E", (203, 35), 8),
+    ("SLA H", (203, 36), 8),
+    ("SLA L", (203, 37), 8),
+    ("SLA (HL)", (203, 38), 15),
+    ("SLA A", (203, 39), 8),
+    ("SLA (IX+0)", (221, 203, 0, 38), 23),
+    ("SLA (IY+0)", (253, 203, 0, 38), 23),
+    ("SLL B", (203, 48), 8),
+    ("SLL C", (203, 49), 8),
+    ("SLL D", (203, 50), 8),
+    ("SLL E", (203, 51), 8),
+    ("SLL H", (203, 52), 8),
+    ("SLL L", (203, 53), 8),
+    ("SLL (HL)", (203, 54), 15),
+    ("SLL A", (203, 55), 8),
+    ("SLL (IX+0)", (221, 203, 0, 54), 23),
+    ("SLL (IY+0)", (253, 203, 0, 54), 23),
+    ("SRA B", (203, 40), 8),
+    ("SRA C", (203, 41), 8),
+    ("SRA D", (203, 42), 8),
+    ("SRA E", (203, 43), 8),
+    ("SRA H", (203, 44), 8),
+    ("SRA L", (203, 45), 8),
+    ("SRA (HL)", (203, 46), 15),
+    ("SRA A", (203, 47), 8),
+    ("SRA (IX+0)", (221, 203, 0, 46), 23),
+    ("SRA (IY+0)", (253, 203, 0, 46), 23),
+    ("SRL B", (203, 56), 8),
+    ("SRL C", (203, 57), 8),
+    ("SRL D", (203, 58), 8),
+    ("SRL E", (203, 59), 8),
+    ("SRL H", (203, 60), 8),
+    ("SRL L", (203, 61), 8),
+    ("SRL (HL)", (203, 62), 15),
+    ("SRL A", (203, 63), 8),
+    ("SRL (IX+0)", (221, 203, 0, 62), 23),
+    ("SRL (IY+0)", (253, 203, 0, 62), 23),
+    ("SUB 0", (214, 0), 7),
+    ("SUB B", (144,), 4),
+    ("SUB C", (145,), 4),
+    ("SUB D", (146,), 4),
+    ("SUB E", (147,), 4),
+    ("SUB H", (148,), 4),
+    ("SUB L", (149,), 4),
+    ("SUB (HL)", (150,), 7),
+    ("SUB A", (151,), 4),
+    ("SUB IXh", (221, 148), 8),
+    ("SUB IXl", (221, 149), 8),
+    ("SUB (IX+0)", (221, 150, 0), 19),
+    ("SUB IYh", (253, 148), 8),
+    ("SUB IYl", (253, 149), 8),
+    ("SUB (IY+0)", (253, 150, 0), 19),
+    ("XOR 0", (238, 0), 7),
+    ("XOR B", (168,), 4),
+    ("XOR C", (169,), 4),
+    ("XOR D", (170,), 4),
+    ("XOR E", (171,), 4),
+    ("XOR H", (172,), 4),
+    ("XOR L", (173,), 4),
+    ("XOR (HL)", (174,), 7),
+    ("XOR A", (175,), 4),
+    ("XOR IXh", (221, 172), 8),
+    ("XOR IXl", (221, 173), 8),
+    ("XOR (IX+0)", (221, 174, 0), 19),
+    ("XOR IYh", (253, 172), 8),
+    ("XOR IYl", (253, 173), 8),
+    ("XOR (IY+0)", (253, 174, 0), 19)
+)
+
+class Instruction:
+    def __init__(self, operation, byte_values):
+        self.operation = operation
+        self.bytes = byte_values
+
 class Z80Test(SkoolKitTestCase):
     def setUp(self):
         SkoolKitTestCase.setUp(self)
@@ -1501,6 +2307,11 @@ class Z80Test(SkoolKitTestCase):
         for operation in invalid_operations:
             self._test_assembly(operation)
 
+    def test_get_timing(self):
+        for op, byte_values, exp_timing in TIMINGS:
+            instruction = Instruction(op, byte_values)
+            self.assertEqual(exp_timing, get_timing(instruction), f'get_timing failed for "{op}"')
+
     @patch.object(components, 'SK_CONFIG', None)
     def test_custom_operand_evaluator(self):
         custom_evaluator = """
